# -*- coding: utf-8 -*-
#COPYRIGHT 2016 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""ArcGIS Pipeline Referencing is a data management application suite
that provides an integrated framework for maintaining pipeline data
with linear referencing systems. The Location Referencing toolbox
provides essential tools for regenerating routes, managing calibration
points and events, and executing quality control checks."""
__all__ = ['AppendEvents', 'ApplyEventBehaviors', 'CreateUPDMGeodatabase',
           'DeriveEventMeasures', 'GenerateEvents', 'GenerateRoutes',
           'OverlayEvents', 'TranslateEventMeasures']
__alias__ = 'locref'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs, gptoolnotsupported
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptoolnotsupported
@gptooldoc('AppendEvents_locref', None)
def AppendEvents(in_dataset=None, in_target_event=None, field_mapping=None, load_type=None, generate_event_ids=None, generate_shapes=None):
    """AppendEvents_locref(in_dataset, in_target_event, field_mapping, {load_type}, {generate_event_ids}, {generate_shapes})

        Appends event records from a table, layer, or feature class into an
        existing ArcGIS Pipeline Referencing event feature class.

     INPUTS:
      in_dataset (Table View):
          The source event records to append.
      in_target_event (Feature Layer):
          The Pipeline Referencing event layer or feature class into which the
          source event records append.
      field_mapping (Field Mappings):
          Controls how the attribute information in the in_dataset's fields is
          transferred to the in_target_event. Because the in_dataset's data is
          appended into an existing event that
          has a predefined schema (field definitions), fields cannot be added or
          removed from the target dataset. While you can set merge rules for
          each output field, the tool ignores those rules.You can use the ArcPy
          FieldMappings class to define this parameter.
      load_type {String}:
          Determines how appended events with measure or temporality overlaps
          with identical Event IDs as Target Event records are loaded into the
          event feature class.

          * ADD-Appends the Input Event records to the Target Event. No changes
          are made to Target Event records.

          * RETIRE_OVERLAPS-Appends the Input Event records to the Target Event
          and retires any records in the Target Event with measure or
          temporality overlaps as the appended events. If the appended event
          eclipses the Target Event, the Target Event will be deleted. This
          option should only be used for linear events.

          * RETIRE_BY_EVENT_ID-Appends the Input Event records to the Target
          Event and retires any records in the Target Event with the same Event
          ID and temporality overlaps as the appended events. If the appended
          event eclipses a Target Event with the same Event ID, the Target Event
          will be deleted.

          * REPLACE_BY_EVENT_ID-Appends the Input Event records to the Target
          Event and deletes any records in the Target Event with the same Event
          ID as the appended events.
      generate_event_ids {Boolean}:
          Indicates whether to generate Event IDs for in_dataset records being
          appended. Generation of Events IDs will only be applied to in_dataset
          records with a Null value for the Event ID field.

          * GENERATE_EVENT_IDS-Generates Event IDs of thein_dataset records
          being appended.

          * NO_GENERATE_EVENT_IDS-Does not generate the Event IDs of the
          in_dataset records being appended. This is the default.
      generate_shapes {Boolean}:
          Indicates whether to regenerate the shapes for the records being
          appended. This parameter is only active when the in_dataset is a
          feature layer or feature class.

          * GENERATE_SHAPES-Regenerates the shapes of the input event features.
          This is the default.

          * NO_SHAPES-Does not regenerate the shapes of the input event
          features."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.AppendEvents_locref(*gp_fixargs((in_dataset, in_target_event, field_mapping, load_type, generate_event_ids, generate_shapes), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('ApplyEventBehaviors_locref', None)
def ApplyEventBehaviors(in_route_features=None):
    """ApplyEventBehaviors_locref(in_route_features)

        Updates the event locations for all event feature classes registered
        with the input network based on the route edit performed.The events
        get updated as per the event behavior configured. To learn
        more about event behavior and configuring event behavior, see  Event
        behaviors.

     INPUTS:
      in_route_features (Feature Layer):
          The ALRS Network for which event locations will be updated."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.ApplyEventBehaviors_locref(*gp_fixargs((in_route_features,), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('CreateUPDMGeodatabase_locref', None)
def CreateUPDMGeodatabase(output_gdb=None, spatial_reference=None, measure_unit=None, xy_tolerance=None, z_tolerance=None, event_registration_network=None, register_p_pipesystem=None):
    """CreateUPDMGeodatabase_locref(output_gdb, spatial_reference, {measure_unit}, {xy_tolerance}, {z_tolerance}, {event_registration_network}, {register_p_pipesystem})

        Populates an existing file or multiuser geodatabase with all feature
        datasets, feature classes, tables, relationship classes, and domains
        needed for the Utility and Pipeline Data Model (UPDM) and creates an
        ALRS with events registered to the chosen network.

     INPUTS:
      output_gdb (Workspace):
          The file or multiuser geodatabase where the UPDM ALRS will be
          created.
      spatial_reference (Spatial Reference):
          The spatial reference for feature datasets and feature classes within
          the geodatabase.
      measure_unit {String}:
          The unit of measure for the M values. This is the unit of measure
          that will be used for the LRS Networks created by the tool.

          * CENTIMETERS-Centimeters is the unit of measure.

          * DECIMETERS-Decimeters is the unit of measure.

          * FEET-Feet is the unit of measure.

          * INCHES-Inches is the unit of measure.

          * KILOMETERS-Kilometers is the unit of measure.

          * METERS-Meters is the unit of measure.

          * MILES-Miles is the unit of measure.

          * MILLIMETERS-Millimeters is the unit of measure.

          * NAUTICAL_MILES-Nautical Miles is the unit of measure.

          * YARDS-Yards is the unit of measure.
      xy_tolerance {Double}:
          The XY tolerance for feature datasets and feature classes within the
          geodatabase. XY resolution will be one-tenth the XY tolerance. M
          tolerance will be equal to the XY tolerance converted to the selected
          M unit of measure. M resolution will be one-tenth the M tolerance.
      z_tolerance {Double}:
          The Z tolerance for feature datasets and feature classes within the
          geodatabase. Z resolution will be one-tenth the Z tolerance.
      event_registration_network {String}:
          The network to which events will be registered. Choose Engineering if
          your events will be registered to the Engineering network and will
          support lines and events that span routes. Choose Continuous if your
          events will be registered to the Continuous network and linear events
          do not need to span routes.

          * CONTINUOUS-Events will be registered to the Continuous network.

          * ENGINEERING-Events will be registered to the Engineering network.
      register_p_pipesystem {Boolean}:
          Determines whether all event feature classes in the network will be
          updated.

          * TRUE-Feature classes within P_PipeSystem will be registered as
          events.

          * FALSE-Feature classes within P_PipeSystem will not be registered as
          events."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateUPDMGeodatabase_locref(*gp_fixargs((output_gdb, spatial_reference, measure_unit, xy_tolerance, z_tolerance, event_registration_network, register_p_pipesystem), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('DeriveEventMeasures_locref', None)
def DeriveEventMeasures(in_route_features=None, update_all_events=None, event_layers=None):
    """DeriveEventMeasures_locref(in_route_features, {update_all_events}, {event_layers;event_layers...})

        Populates and updates the Derived RouteID and measure values on point
        and line events with those fields configured and enabled.

     INPUTS:
      in_route_features (Feature Layer):
          The LRS Network containing the events with derived network RouteID
          and measure fields configured.
      update_all_events {Boolean}:
          Determines whether all event feature classes in the network will be
          updated.

          * UPDATE_ALL-Updates all events in the network selected in
          in_route_features. This is the default.

          * UPDATE_SOME-Does not update all events in the network selected in
          the in_route_features. Allows the user to select individual event
          layers in the event layers parameter below.
      event_layers {Feature Layer}:
          The event layers that will have derived network RouteID and measure
          fields updated."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.DeriveEventMeasures_locref(*gp_fixargs((in_route_features, update_all_events, event_layers), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('GenerateEvents_locref', None)
def GenerateEvents(in_event_layer=None):
    """GenerateEvents_locref(in_event_layer)

        Re-creates shapes for event features registered with an LRS Network.
        If the event has derived network route and measure fields configured,
        the tool will update those fields when executed.

     INPUTS:
      in_event_layer (Feature Layer):
          The event for which shapes will be regenerated."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateEvents_locref(*gp_fixargs((in_event_layer,), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('GenerateRoutes_locref', None)
def GenerateRoutes(in_route_features=None, record_calibration_changes=None):
    """GenerateRoutes_locref(in_route_features, {record_calibration_changes})

        Re-creates shapes and applies calibration changes for route features
        in an LRS Network.Additionally, the tool can determine calibration
        changes on routes so
        event behaviors can be processed. These calibration changes can be due
        to adding, editing, or deleting calibration points outside the
        Location Referencing tools in ArcGIS Pro.

     INPUTS:
      in_route_features (Feature Layer):
          The LRS Network for which route shapes will be regenerated and
          calibration changes will be applied.
      record_calibration_changes {Boolean}:
          * RECORD_CALIBRATION_CHANGES-Any calibration points created, modified,
          or deleted outside the ArcGIS Pipeline Referencing tools will be
          applied to the routes in the network and event behaviors will be
          applied the next time Apply Event Behaviors is run.

          * NO_RECORD_CALIBRATION_CHANGES-Calibration changes will be applied to
          the routes in the LRS Network, but no event behaviors will be applied.
          This is the default."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.GenerateRoutes_locref(*gp_fixargs((in_route_features, record_calibration_changes), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('OverlayEvents_locref', None)
def OverlayEvents(in_route_features=None, event_layers=None, output_dataset=None, include_geometry=None, network_fields=None):
    """OverlayEvents_locref(in_route_features, event_layers;event_layers..., output_dataset, {include_geometry}, {network_fields;network_fields...})

        Overlays one or more linear event feature layers against a target
        network and outputs a feature class or table that represents the
        dynamic segmentation of the inputs.

     INPUTS:
      in_route_features (Feature Layer):
          The target network against which the event layers will be dynamically
          segmented.
      event_layers (Feature Layer):
          The event layers to be dynamically segmented together against a
          target network.
      include_geometry {Boolean}:
          Determines whether the output_dataset will include a geometry column.

          * EXCLUDE_GEOMETRY-Will not include the event geometry in the
          output_dataset. Event records will be stored as a table. This is the
          default.

          * INCLUDE_GEOMETRY-Will include the event geometry in the
          output_dataset. Event records will be stored as a feature class.
      network_fields {Field}:
          Fields from the network layer that will be included in the output.

     OUTPUTS:
      output_dataset (Table):
          The table or feature class to be created, containing the output event
          records."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.OverlayEvents_locref(*gp_fixargs((in_route_features, event_layers, output_dataset, include_geometry, network_fields), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('TranslateEventMeasures_locref', None)
def TranslateEventMeasures(in_source_event=None, in_target_route_features=None, out_target_event=None, in_concurrent_route_matching=None):
    """TranslateEventMeasures_locref(in_source_event, in_target_route_features, out_target_event, {in_concurrent_route_matching})

        Translates the measures (m-values) of events from one LRM (Linear
        Referencing Method) to another. The output is a new feature class.This
        tool brings together event data provided by various business
        units that may use different LRMs. These LRMs may have different route
        IDs and m-values that require translation between LRMs.

     INPUTS:
      in_source_event (Feature Layer):
          The source event layer to be translated.
      in_target_route_features (Feature Layer):
          The route network to which the source events will be translated.
      in_concurrent_route_matching {String}:
          The method to determine which route to translate the event onto when
          concurrent routes exist in the target network. This selected method is
          only applied when the location of the event translation in the target
          network has concurrent routes.

          * ANY-The source event is translated against the first of the
          concurrent routes at the source event location in the target network.

          * ROUTE_ID-The RouteID of the source event is compared to the RouteIDs
          of concurrent routes at the source event location in the target
          network. The source event will translate based on RouteIDs matching in
          the source event and target network. The RouteIDs of the source event
          and target network must be an exact match for this method to correctly
          translate the event.

          * ALL-The source event is translated against all the concurrent routes
          at that location in the target network.

     OUTPUTS:
      out_target_event (Feature Class):
          The translated event to be created."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.TranslateEventMeasures_locref(*gp_fixargs((in_source_event, in_target_route_features, out_target_event, in_concurrent_route_matching), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject, gptoolnotsupported