# -*- coding: utf-8 -*-
#COPYRIGHT 2016 ESRI
#
#TRADE SECRETS: ESRI PROPRIETARY AND CONFIDENTIAL
#Unpublished material - all rights reserved under the
#Copyright Laws of the United States.
#
#For additional information, contact:
#Environmental Systems Research Institute, Inc.
#Attn: Contracts Dept
#380 New York Street
#Redlands, California, USA 92373
#
#email: contracts@esri.com
r"""The Space Time Pattern Mining toolbox contains statistical tools for
analyzing data distributions and patterns in the context of both space
and time. It includes a toolset for visualizing the data stored in the
space-time netCDF cube in both 2D and 3D."""
__all__ = ['CreateSpaceTimeCube', 'EmergingHotSpotAnalysis',
           'LocalOutlierAnalysis', 'VisualizeSpaceTimeCube2D',
           'VisualizeSpaceTimeCube3D']
__alias__ = 'stpm'
from arcpy.geoprocessing._base import gptooldoc, gp, gp_fixargs, gptoolnotsupported
from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject

# Tools
@gptoolnotsupported
@gptooldoc('CreateSpaceTimeCube_stpm', None)
def CreateSpaceTimeCube(in_features=None, output_cube=None, time_field=None, template_cube=None, time_step_interval=None, time_step_alignment=None, reference_time=None, distance_interval=None, summary_fields=None, aggregation_shape_type=None):
    """CreateSpaceTimeCube_stpm(in_features, output_cube, time_field, {template_cube}, {time_step_interval}, {time_step_alignment}, {reference_time}, {distance_interval}, {summary_fields;summary_fields...}, {aggregation_shape_type})

        Summarizes a set of points into a netCDF data structure by
        aggregating them into space-time bins. Within each bin, the points are
        counted and specified attributes are aggregated. For all bin
        locations, the trend for counts and summary field values are
        evaluated.

     INPUTS:
      in_features (Feature Layer):
          The input point feature class to be aggregated into space-time bins.
      time_field (Field):
          The field containing the date and time (timestamp) for each point.
          This field must be of type Date.
      template_cube {File}:
          A reference space-time cube used to define the output_cube extent of
          analysis, bin dimensions, and bin alignment.  The time_step_interval,
          distance_interval, and reference_time values are also obtained from
          the template cube.  This template cube must be a netCDF (.nc) file
          that has been created using this tool.
      time_step_interval {Time unit}:
          The number of seconds,  minutes, hours, days, weeks, or years that
          will represent a single time step. All points within the same
          time_step_interval and distance_interval will be aggregated. (When a
          template_cube is provided, this parameter is ignored, and the
          time_step_interval value is obtained from the template cube). Examples
          of valid entries for this parameter are 1 Weeks, 13 Days, or 1 Months.
      time_step_alignment {String}:
          Defines how aggregation will occur based on a given
          time_step_interval. If a template_cube is provided, the
          time_step_alignment associated with the template_cube overrides this
          parameter setting and the time_step_alignment of the template_cube is
          used.

          * END_TIME-Time steps align to the last time event and aggregate back
          in time.

          * START_TIME-Time steps align to the first time event and aggregate
          forward in time.

          * REFERENCE_TIME-Time steps align to a particular date/time that you
          specify. If all points in the input features have a timestamp larger
          than the reference time you provide (or it falls exactly on the start
          time of the input features), the time-step interval will begin with
          that reference time and aggregate forward in time (as occurs with a
          START_TIME alignment). If all points in the input features have a
          timestamp smaller than the reference time you provide (or it falls
          exactly on the end time of the input features), the time-step interval
          will end with that reference time and aggregate backward in time (as
          occurs with an END_TIME alignment). If the reference time you provide
          is in the middle of the time extent of your data, a time-step interval
          will be created ending with the reference time provided (as occurs
          with an END_TIME alignment); additional intervals will be created both
          before and after the reference time until the full time extent of your
          data is covered.
      reference_time {Date}:
          The date/time to use to align the time-step intervals. If you want to
          bin your data weekly from Monday to Sunday, for example, you could set
          a reference time of Sunday at midnight to ensure bins break between
          Sunday and Monday at midnight.  (When a template_cube is provided,
          this parameter is ignored and the reference_time is based on the
          template_cube.)
      distance_interval {Linear Unit}:
          The size of the bins used to aggregate the in_features. All points
          that fall within the same distance_interval and time_step_interval
          will be aggregated. When aggregating into a hexagon grid, this
          distance is used as the height to construct the hexagon polygons.
          (When a template_cube is provided, this parameter is ignored and the
          distance interval value will be based on the template_cube.)
      summary_fields {Value Table}:
          The numeric field containing attribute values used to calculate the
          specified statistic when aggregating into a space-time cube. Multiple
          statistic and field combinations may be specified. Null values are
          excluded from all statistical calculations.Available statistic types
          are:

          * SUM-Adds the total value for the specified field within each bin.

          * MEAN-Calculates the average for the specified field wintin each bin.

          * MIN-Finds the smallest value for all records of the specified field
          within each bin.

          * MAX-Finds the largest value for all records of the specified field
          withtin each bin.

          * STD-Finds the standard deviation on values in the specified field
          within each bin.

          * MEDIAN-Finds the sorted middle value of all records of the specified
          field within each bin.
          Available fill types are:

          * ZEROS-Fills empty bins with zeros.

          * SPATIAL_NEIGHBORS-Fills empty bins with the average value of spatial
          neighbors

          * SPACE_TIME_NEIGHBORS-Fills empty bins with the average value of
          space time neighbors.

          * TEMPORAL_TREND-Fills empty bins using an interpolated univariate
          spline algorithm.
          Note: Null values present in any of the summary fields will result in
          those features being excluded from analysis. If having the count of
          points in each bin is part of your analysis strategy, you may want to
          consider creating separate cubes, one for the count (without summary
          fields) and one for summary fields. If the set of null values is
          different for each summary field, you may also consider creating a
          separate cube for each summary field.
      aggregation_shape_type {String}:
          The shape of the polygon mesh the input feature point data will be
          aggregated into.

          * FISHNET_GRID-The input features will be aggregated into a grid of
          square (fishnet) cells.

          * HEXAGON_GRID-The input features will be aggregated into a grid of
          hexagonal cells.

     OUTPUTS:
      output_cube (File):
          The output netCDF data cube that will be created to contain counts
          and summaries of the input feature point data."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.CreateSpaceTimeCube_stpm(*gp_fixargs((in_features, output_cube, time_field, template_cube, time_step_interval, time_step_alignment, reference_time, distance_interval, summary_fields, aggregation_shape_type), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('EmergingHotSpotAnalysis_stpm', None)
def EmergingHotSpotAnalysis(in_cube=None, analysis_variable=None, output_features=None, neighborhood_distance=None, neighborhood_time_step=None, polygon_mask=None):
    """EmergingHotSpotAnalysis_stpm(in_cube, analysis_variable, output_features, {neighborhood_distance}, {neighborhood_time_step}, {polygon_mask})

        Identifies trends in the clustering of point densities (counts) or
        summary fields in a space-time cube created using the Create Space
        Time Cube tool. Categories include new, consecutive, intensifying,
        persistent, diminishing, sporadic, oscillating, and historical hot and
        cold spots.

     INPUTS:
      in_cube (File):
          The netCDF cube to be analyzed. This file must have an (.nc)
          extension and must have been created using the Create Space Time Cube
          tool.
      analysis_variable (String):
          The numeric variable in the netCDF file you want to analyze.
      neighborhood_distance {Linear Unit}:
          The spatial extent of the analysis neighborhood. This value
          determines which features are analyzed together in order to assess
          local space-time clustering.
      neighborhood_time_step {Long}:
          The number of time-step intervals to include in the analysis
          neighborhood. This value determines which features are analyzed
          together in order to assess local space-time clustering.
      polygon_mask {Feature Layer}:
          A polygon feature layer with one or more polygons defining the
          analysis study area. You would use a polygon analysis mask to exclude
          a large lake from the analysis, for example. Bins defined in the Input
          Space Time Cube that fall outside of the mask will not be included in
          the analysis.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a two-
          dimensional map representation of the hot and cold spot trends in your
          data. It will show, for example, any new or intensifying hot spots."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.EmergingHotSpotAnalysis_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, neighborhood_distance, neighborhood_time_step, polygon_mask), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('LocalOutlierAnalysis_stpm', None)
def LocalOutlierAnalysis(in_cube=None, analysis_variable=None, output_features=None, neighborhood_distance=None, neighborhood_time_step=None, number_of_permutations=None, polygon_mask=None):
    """LocalOutlierAnalysis_stpm(in_cube, analysis_variable, output_features, {neighborhood_distance}, {neighborhood_time_step}, {number_of_permutations}, {polygon_mask})

        Identifies statistically significant clusters and outliers in the
        context of both space and time. This tool is a space-time
        implementation of the Anselin Local Moran's I statistic.

     INPUTS:
      in_cube (File):
          The netCDF cube to be analyzed. This file must have an (.nc)
          extension and must have been created using the Create Space Time Cube
          tool.
      analysis_variable (String):
          The numeric variable in the netCDF file you want to analyze.
      neighborhood_distance {Linear Unit}:
          The spatial extent of the analysis neighborhood. This value
          determines which features are analyzed together in order to assess
          local space-time clustering.
      neighborhood_time_step {Long}:
          The number of time-step intervals to include in the analysis
          neighborhood. This value determines which features are analyzed
          together in order to assess local space-time clustering.
      number_of_permutations {Long}:
          The number of random permutations for the calculation of pseudo
          p-values. The default number of permutations is 499. If you choose 0
          permutations, the standard p-value is calculated.

          * 0-Permutations are not used and a standard p-value is calculated.

          * 99-With 99 permutations, the smallest possible pseudo p-value is
          0.01 and all other pseudo p-values will be even multiples of this
          value.

          * 199-With 199 permutations, the smallest possible pseudo p-value is
          0.005 and all other pseudo p-values will be even multiples of this
          value.

          * 499-With 499 permutations, the smallest possible pseudo p-value is
          0.002 and all other pseudo p-values will be even multiples of this
          value.

          * 999-With 999 permutations, the smallest possible pseudo p-value is
          0.001 and all other pseudo p-values will be even multiples of this
          value.

          * 9999-With 9999 permutations, the smallest possible pseudo p-value is
          0.0001 and all other pseudo p-values will be even multiples of this
          value.
      polygon_mask {Feature Layer}:
          A polygon feature layer with one or more polygons defining the
          analysis study area. You would use a polygon analysis mask to exclude
          a large lake from the analysis, for example. Bins defined in the Input
          Space Time Cube that fall outside of the mask will not be included in
          the analysis.

     OUTPUTS:
      output_features (Feature Class):
          The output feature class containing locations that were considered
          statistically significant clusters or outliers."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.LocalOutlierAnalysis_stpm(*gp_fixargs((in_cube, analysis_variable, output_features, neighborhood_distance, neighborhood_time_step, number_of_permutations, polygon_mask), True)))
        return retval
    except Exception as e:
        raise e


# Utilities toolset
@gptoolnotsupported
@gptooldoc('VisualizeSpaceTimeCube2D_stpm', None)
def VisualizeSpaceTimeCube2D(in_cube=None, cube_variable=None, display_theme=None, output_features=None):
    """VisualizeSpaceTimeCube2D_stpm(in_cube, cube_variable, display_theme, output_features)

        Visualizes the variables stored in a netCDF cube and the results
        generated by the Space Time Pattern Mining tools. Output from this
        tool is a two-dimensional representation uniquely rendered based on
        the variable and theme chosen.

     INPUTS:
      in_cube (File):
          The netCDF cube that contains the variable to be displayed. This file
          must have an .nc extension and must have been created using the Create
          Space Time Cube  tool.
      cube_variable (String):
          The numeric variable in the netCDF cube that you want to explore. The
          cube will always contain the COUNT variable. Any Summary Fields in the
          cube will also be available if they were included in the cube creation
          process.
      display_theme (String):
          The characteristic of the Cube Variable that you want to display.
          Locations with data and Trends will always be available with every
          cube. Hot and cold spot trends and Emerging Hot Spot Analysis results
          will only be available after Emerging Hot Spot Analysis has been run
          on the selected Cube Variable. Percentage of local outliers, Local
          outlier in the most recent time period, Local Outlier Analysis
          results, and Locations without spatial neighbors are only available
          when you have run the Local Outlier Analysis tool.  Number of
          estimated bins and Locations excluded from analysis will only be
          available for those Summary Fields that were included in the cube
          creation process.

          * LOCATIONS_WITH_DATA-Displays all locations that contain data for the
          chosen variable

          * TRENDS-The trend of values at each location using the Mann-Kendall
          statistic

          * HOT_AND_COLD_SPOT_TRENDS-The trend of z-scores at each location
          using the Mann-Kendall statistic

          * EMERGING_HOT_SPOT_ANALYSIS_RESULTS-Displays the results of Emerging
          Hot Spot Analysis for the Cube Variable chosen

          * LOCAL_OUTLIER_ANALYSIS_RESULTS-Displays the results of Local Outlier
          Analysis for the Cube Variable chosen

          * PERCENTAGE_OF_LOCAL_OUTLIERS-Displays the total percentage of
          outliers at each location

          * LOCAL_OUTLIER_IN_MOST_RECENT_TIME_PERIOD-Displays all outliers that
          are present in the most recent time period

          * LOCATIONS_WITHOUT_SPATIAL_NEIGHBORS-Displays all locations that have
          no spatial neighbors and, as a result, are relying on temporal
          neighbors for analysis

          * NUMBER_OF_ESTIMATED_BINS-The number of bins that were estimated for
          each location

          * LOCATIONS_EXCLUDED_FROM_ANALYSIS-The locations that were excluded
          from analysis because they had empty bins that did not meet the
          criteria for estimation

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a two-
          dimensional map representation of the display variable chosen."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.VisualizeSpaceTimeCube2D_stpm(*gp_fixargs((in_cube, cube_variable, display_theme, output_features), True)))
        return retval
    except Exception as e:
        raise e

@gptoolnotsupported
@gptooldoc('VisualizeSpaceTimeCube3D_stpm', None)
def VisualizeSpaceTimeCube3D(in_cube=None, cube_variable=None, display_theme=None, output_features=None):
    """VisualizeSpaceTimeCube3D_stpm(in_cube, cube_variable, display_theme, output_features)

        Visualizes the variables stored in a netCDF cube created with the
        Space Time Pattern Mining tools. Output from this tool is a three-
        dimensional representation uniquely rendered based on the variable and
        theme chosen.

     INPUTS:
      in_cube (File):
          The netCDF cube that contains the variable to be displayed. This file
          must have an .nc extension and must have been created using the Create
          Space Time Cube tool.
      cube_variable (String):
          The numeric variable in the netCDF cube that you want to explore. The
          cube will always contain the COUNT variable. Any Summary Fields in the
          cube will also be available if they were included in the cube creation
          process.
      display_theme (String):
          The characteristic of the Cube Variable that you want to display.
          Value will always contain an option for every cube. Estimated bins are
          only available for the Summary Fields that were included in the cube
          creation process. Hot and cold spot results will only be available for
          Cube Variables for which Emerging Hot Spot Analysis has been run.
          Cluster and outlier results will only be available for Cube Variables
          for which Local Outlier Analysis has been run.

          * VALUE-The numeric value of the cube variable chosen

          * HOT_AND_COLD_SPOT_RESULTS-The statistical significance of each bin
          based on the space-time hot spot analysis run in Emerging Hot Spot
          Analysis

          * ESTIMATED_BINS-Displays the bins with estimated values

          * LOCAL_OUTLIER_RESULTS-The cluster or outlier type (COType) assigned
          to each bin from the analysis run in Local Outlier Analysis

     OUTPUTS:
      output_features (Feature Class):
          The output feature class results. This feature class will be a three-
          dimensional map representation of the display variable chosen that can
          be displayed in a 3D scene."""
    from arcpy.geoprocessing._base import gp, gp_fixargs
    from arcpy.arcobjects.arcobjectconversion import convertArcObjectToPythonObject
    try:
        retval = convertArcObjectToPythonObject(gp.VisualizeSpaceTimeCube3D_stpm(*gp_fixargs((in_cube, cube_variable, display_theme, output_features), True)))
        return retval
    except Exception as e:
        raise e


# End of generated toolbox code
del gptooldoc, gp, gp_fixargs, convertArcObjectToPythonObject, gptoolnotsupported